#include "MCManager.h"
#include "../entities/Doctor.h"
#include "../entities/Patient.h"
#include <Foundation/NSException.h>
#include <Foundation/NSEnumerator.h>

/**
  * Egeszsegkozpont manager osztaly.
  */

@implementation MCManager 

/**
 * Inicializalas.
 */
-(MCManager*) initWithParams: (NSString*) newDoctorResourcePath : (NSString*) newDiseaseResourcePath : (NSString*) newPatientResourcePath{
	
	doctorResourcePath = newDoctorResourcePath;
	diseaseResourcePath = newDiseaseResourcePath;
	patientResourcePath = newPatientResourcePath;
}


/**
 * Orvosok es kezelt betegsegek betoltese fajlbol.
 */
- (void) loadDoctors {

	if (fm == nil)
		fm = [NSFileManager defaultManager];

	BOOL i = YES;
        if ([fm fileExistsAtPath:doctorResourcePath isDirectory:&i]) {
		
		NSArray* lines;
		lines = [[[NSString stringWithContentsOfFile: doctorResourcePath] stringByStandardizingPath] componentsSeparatedByString:@"\n"];

		NSEnumerator *e = [lines objectEnumerator];
		NSString* line;

		doctors = [NSMutableArray array];

		while ((line = [e nextObject]) && ([line length] != 0)) {
			
			Doctor* doc = [[Doctor alloc] init];

			NSArray* details = [line componentsSeparatedByString:@";"];

			NSMutableArray* dis = [[NSMutableArray alloc] init];

			int idx = 4;
			for (idx; idx<[details count];++idx) 
				[dis addObject: [details objectAtIndex:idx]];
			
		        [doc initDoctorWithParams: 
					[details objectAtIndex:0]
					: [details objectAtIndex:1]
					: [[details objectAtIndex:2] intValue]
					: [details objectAtIndex:3]
					: dis ];

			[doctors addObject: doc];
		}

	}else {
		NSException* myException = [NSException exceptionWithName:@"FileNotFoundException" reason:@"Cannot find resource file." userInfo:nil];
		@throw myException;
	}
}

/*
 * Betegsegek es gyogyszerek betoltese fajlbol.
 */
- (void) loadDiseases {

	if (fm == nil)
		fm = [NSFileManager defaultManager];

	BOOL i = YES;
	if ([fm fileExistsAtPath:diseaseResourcePath isDirectory:&i]) {
		
		NSArray* lines;
		lines = [[[NSString stringWithContentsOfFile: diseaseResourcePath] stringByStandardizingPath] componentsSeparatedByString:@"\n"];
	
		NSEnumerator *e = [lines objectEnumerator];
		NSString* line;

		diseases = [NSMutableArray array];

		 while ((line = [e nextObject]) && ([line length] != 0)) {
		 
			Disease* dis = [[Disease alloc] init];

			NSArray* details = [line componentsSeparatedByString:@";"];
                        NSArray* meds = [details subarrayWithRange:NSMakeRange(1, [details count] - 1) ];
                        [dis initWithParams: (NSString*)[details objectAtIndex:0]  : meds ];

			[diseases addObject : dis];

		}

	}else {
		NSException* myException = [NSException exceptionWithName:@"FileNotFoundException" reason:@"Cannot find resource file." userInfo:nil];
		@throw myException;
	}
}


/*
 * Betegek s betegsegek betoltese fajlbol.
 */
- (void) loadPatients {

	if (fm == nil)
		fm = [NSFileManager defaultManager];

	BOOL i = YES;
	if ([fm fileExistsAtPath:patientResourcePath isDirectory:&i]) {
		
		NSArray* lines;
		lines = [[[NSString stringWithContentsOfFile: patientResourcePath] stringByStandardizingPath] componentsSeparatedByString:@"\n"];

		patients = [NSMutableArray array];
	
		NSEnumerator *e = [lines objectEnumerator];
		NSString* line;

		while ((line = [e nextObject]) && ([line length] != 0)) {
		 			
			Patient* pat = [[Patient alloc] init];

			NSArray* details = [line componentsSeparatedByString:@";"];
			Disease* dis;
			

			int i = 0;
			for (i; i<[diseases count]; ++i) {

				if ([[(Disease*)[diseases objectAtIndex:i]  getName] isEqualToString: (NSString*)[details objectAtIndex:4]]) {

					dis = (Disease*)[diseases objectAtIndex:i];
					break;
				}
			}

                        [pat initPatientWithParams: (NSString*)[details objectAtIndex:0]  
						: (NSString*)[details objectAtIndex:1]  
						: [[details objectAtIndex:2] intValue] 
						: (NSString*)[details objectAtIndex:3]  
						: dis];

                        [patients addObject: pat];
		}

	}else {
		NSException* myException = [NSException exceptionWithName:@"FileNotFoundException" reason:@"Cannot find resource file." userInfo:nil];
		@throw myException;
	}
}


/**
  * Betegek beosztasa orvosokhoz.
  */
- (void) organizePatients {

        patientsToDoctorsMap = [[NSMutableDictionary alloc] init];

	int i =0;
	for (i; i<[patients count]; ++i) {

		Disease* dis =[[patients objectAtIndex:i] getDisease];
		BOOL ok = NO;
		
		int docIdx = 0;
		for (docIdx; docIdx<[doctors count]; ++docIdx) {

			NSArray* treated = [[doctors objectAtIndex:docIdx] getDiseases];
			Disease* d;
			int disIdx = 0;
			for (disIdx; disIdx<[treated count]; ++disIdx) {
				
				d = (Disease*)[treated objectAtIndex:disIdx]; 
				if ([ [dis getName] isEqualToString: (NSString*)d]) {
					ok = YES;
					break;
				}
			}

			if (ok==YES) break;
		}

		[patientsToDoctorsMap setObject: [[doctors objectAtIndex:docIdx] getName] forKey: [[patients objectAtIndex:i] getName]];
	}
}


/**
  * Betegek ellatasa.
  */
- (void) treatPatients {

	NSEnumerator *enumerator = [patientsToDoctorsMap keyEnumerator];
	id key;
	while (key = [enumerator nextObject]) {

		printf("Dr.%s =>  %s\n", [[patientsToDoctorsMap objectForKey:key] cString], [key cString]);
	}
}


- (void) listDoctors {

	if ([doctors count] != 0) {

		printf("%s\n", "Doctors: ");
		
		int i = 0;
		for (i; i<[doctors count]; ++i)
			printf("\t%s\n", [[[doctors objectAtIndex:i] getDescription] cString]);
	}
}


- (void) listPatients {
	
        if ([patients count] != 0) {

	        printf("%s\n", "Patients: ");

	        int i = 0;
	        for (i; i<[patients count]; ++i)
		        printf("\t%s\n", [[[patients objectAtIndex:i] getDescription] cString]);
	}
}

@end
