//
//  HelloWorldLayer.mm
//  MonkeyRunner
//
//  Created by Ákos Belákovics on 5/18/12.
//  Copyright __MyCompanyName__ 2012. All rights reserved.
//


// Import the interfaces
#import "MonkeyWorldLayer.h"

//Pixel to metres ratio. Box2D uses metres as the unit for measurement.
//This ratio defines how many pixels correspond to 1 Box2D "metre"
//Box2D is optimized for objects of 1x1 metre therefore it makes sense
//to define the ratio so that your most common object type is 1x1 metre.
#define PTM_RATIO    32
#define FLOOR_HEIGHT 30.0f
#define MIN_SCALE 0.5f
#define MAX_SCALE 2.0f

// enums that will be used as tags
enum {
	kTagTileMap = 1,
	kTagBatchNode = 1,
	kTagAnimation1 = 1,
};

@interface MonkeyWorldLayer()

- (void)resetGame;
- (void)createBullets:(int)count;
- (BOOL)attachBullet;
- (void)createTargets;
- (void)createTarget:(NSString*)imageName atPosition:(CGPoint)position rotation:(CGFloat)rotation isCircle:(BOOL)aCircle isStatic:(BOOL)aStatic isMonkey:(BOOL)aMonkey;

- (void)zoomLayer:(float)zoomScale;
- (void)moveBoard:(CGPoint)translation from:(CGPoint)lastLocation;

@end

// MonkeyWorldLayer implementation
@implementation MonkeyWorldLayer

+(CCScene *) scene
{
	// 'scene' is an autorelease object.
	CCScene *scene = [CCScene node];
	
	// 'layer' is an autorelease object.
	MonkeyWorldLayer *layer = [MonkeyWorldLayer node];
	
	// add layer as a child to scene
	[scene addChild: layer];
	
	// return the scene
	return scene;
}

// on "init" you need to initialize your instance
-(id) init
{
	// always call "super" init
	// Apple recommends to re-assign "self" with the "super" return value
	if( (self=[super init])) {
		
		// enable touches
		self.isTouchEnabled = YES;
	
		// screen size
		CGSize screenSize = [CCDirector sharedDirector].winSize;
		CCLOG(@"Screen width %0.2f screen height %0.2f",screenSize.width,screenSize.height);
		
		//[[CCTouchDispatcher sharedDispatcher] addStandardDelegate:self priority:0];
		
		UIPanGestureRecognizer *pan= [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(handlePanFrom:)];
		UIPinchGestureRecognizer *pinch = [[UIPinchGestureRecognizer alloc] initWithTarget:self action:@selector(handlePinchFrom:)];
		
		pan.minimumNumberOfTouches= 2;
		
		[[[CCDirector sharedDirector] openGLView] addGestureRecognizer:pan];
		[[[CCDirector sharedDirector] openGLView] addGestureRecognizer:pinch];
		
		checkCollision= NO;
		
		// Define the gravity vector.
		b2Vec2 gravity;
		gravity.Set(0.0f, -10.0f);
		
		// Do we want to let bodies sleep?
		// This will speed up the physics simulation
		bool doSleep = true;
		
		// Construct a world object, which will hold and simulate the rigid bodies.
		world = new b2World(gravity, doSleep);
		
		world->SetContinuousPhysics(true);
		
		// Debug Draw functions
		m_debugDraw = new GLESDebugDraw( PTM_RATIO );
		world->SetDebugDraw(m_debugDraw);
		
		uint32 flags = 0;
		flags += b2DebugDraw::e_shapeBit;
//		flags += b2DebugDraw::e_jointBit;
//		flags += b2DebugDraw::e_aabbBit;
//		flags += b2DebugDraw::e_pairBit;
//		flags += b2DebugDraw::e_centerOfMassBit;
		m_debugDraw->SetFlags(flags);
		
		// Completely transparent base to attach things that do scroll or zoom
/*		zoombase = [CCLayerColor layerWithColor:ccc4(0,0,0,0)];
		zoombase.position = ccp(0, 0);
		[self addChild:zoombase];		
*/		
		// Initialize the zoombase position
		ZB_last_posn = self.position;
		
		CCSprite *sprite = [CCSprite spriteWithFile:@"grassfront.png"];
		sprite.anchorPoint = CGPointZero;
		sprite.position = CGPointMake(0, FLOOR_HEIGHT - 30);
		[self addChild:sprite z:1];
		
		sprite = [CCSprite spriteWithFile:@"grassfront.png"];
		sprite.anchorPoint = CGPointZero;
		sprite.position = CGPointMake(960.0f, FLOOR_HEIGHT - 30);
		[self addChild:sprite z:1];
		
		sprite = [CCSprite spriteWithFile:@"jungle.png"];
        sprite.anchorPoint = CGPointZero;
        [self addChild:sprite z:-1];

		sprite = [CCSprite spriteWithFile:@"jungle.png"];
        sprite.anchorPoint = CGPointZero;
		sprite.position = CGPointMake(960.0f, 0);
        [self addChild:sprite z:-1];
		
		sprite = [CCSprite spriteWithFile:@"monkey.png"];
        sprite.anchorPoint = CGPointZero;
        sprite.position = CGPointMake(180.0f, FLOOR_HEIGHT);
        [self addChild:sprite z:1];
		
		sprite = [CCSprite spriteWithFile:@"catapult_base_2.png"];
        sprite.anchorPoint = CGPointZero;
        sprite.position = CGPointMake(181.0f, FLOOR_HEIGHT);
        [self addChild:sprite z:1];
		
		sprite = [CCSprite spriteWithFile:@"catapult_base.png"];
        sprite.anchorPoint = CGPointZero;
        sprite.position = CGPointMake(180.0f, FLOOR_HEIGHT);
        [self addChild:sprite z:3];
		
		// Define the ground body.
		b2BodyDef groundBodyDef;
		groundBodyDef.position.Set(0, 0); // bottom-left corner
		
		// Call the body factory which allocates memory for the ground body
		// from a pool and creates the ground box shape (also from a pool).
		// The body is also added to the world.
		groundBody = world->CreateBody(&groundBodyDef);
		
		// Define the ground box shape.
		b2PolygonShape groundBox;		
		
		// bottom
		groundBox.SetAsEdge(b2Vec2(0,FLOOR_HEIGHT/PTM_RATIO), b2Vec2(screenSize.width*4.0f/PTM_RATIO,FLOOR_HEIGHT/PTM_RATIO));
		groundBody->CreateFixture(&groundBox, 0);
		
		// top
		groundBox.SetAsEdge(b2Vec2(0,screenSize.height*4.0f/PTM_RATIO), b2Vec2(screenSize.width*4.0f/PTM_RATIO,screenSize.height*4.0f/PTM_RATIO));
		groundBody->CreateFixture(&groundBox, 0);
		
		// left
		groundBox.SetAsEdge(b2Vec2(0,screenSize.height/PTM_RATIO), b2Vec2(0,0));
		groundBody->CreateFixture(&groundBox, 0);
		
		// right
		//groundBox.SetAsEdge(b2Vec2(screenSize.width*4.0f/PTM_RATIO,screenSize.height*2.0f/PTM_RATIO), b2Vec2(screenSize.width*4.0f/PTM_RATIO,0));
		//groundBody->CreateFixture(&groundBox, 0);
		
		
		// Create the catapult's arm
        //
        CCSprite *arm = [CCSprite spriteWithFile:@"catapult_arm.png"];
		arm.rotation= CC_DEGREES_TO_RADIANS(-120);
		//arm.anchorPoint= CGPointZero;
        [self addChild:arm z:2];
        
        b2BodyDef armBodyDef;
        armBodyDef.type = b2_dynamicBody;
        armBodyDef.linearDamping = 1;
        armBodyDef.angularDamping = 1;
		armBodyDef.angle= CC_DEGREES_TO_RADIANS(45);
        armBodyDef.position.Set(180.0f/PTM_RATIO,(FLOOR_HEIGHT+70.0f)/PTM_RATIO);
        armBodyDef.userData = arm;
        armBody = world->CreateBody(&armBodyDef);
        
        b2PolygonShape armBox;
        b2FixtureDef armBoxDef;
        armBoxDef.shape = &armBox;
        armBoxDef.density = 0.3F;
        armBox.SetAsBox(11.0f/PTM_RATIO, 91.0f/PTM_RATIO);
        armFixture = armBody->CreateFixture(&armBoxDef);
		
		// Create a joint to fix the catapult to the floor.
		//
		b2RevoluteJointDef armJointDef;
		armJointDef.Initialize(groundBody, armBody, b2Vec2(250.0f/PTM_RATIO, FLOOR_HEIGHT/PTM_RATIO));
		armJointDef.enableMotor = true;
		armJointDef.enableLimit = true;
		armJointDef.motorSpeed  = -200; //-1260;
		armJointDef.lowerAngle  = CC_DEGREES_TO_RADIANS(0);
		armJointDef.upperAngle  = CC_DEGREES_TO_RADIANS(40);
		armJointDef.maxMotorTorque = 700; //4800;
		
		armJoint = (b2RevoluteJoint*)world->CreateJoint(&armJointDef);
		
		[self schedule: @selector(tick:)];

		[self performSelector:@selector(resetGame) withObject:nil afterDelay:0.5f];
				
		// Collision
		contactListener = new MyContactListener();
		world->SetContactListener(contactListener);
		
	}
	return self;
}

- (void)createTargets
{
    [trees release];
    [monkeys release];
    trees = [[NSMutableSet alloc] init];
    monkeys = [[NSMutableSet alloc] init];

    [self createTarget:@"monkey_.png" atPosition:CGPointMake(1405.0, FLOOR_HEIGHT) rotation:0.0f isCircle:NO isStatic:NO isMonkey:YES];
}

- (void)createTarget:(NSString*)imageName atPosition:(CGPoint)position rotation:(CGFloat)rotation isCircle:(BOOL)aCircle isStatic:(BOOL)aStatic isMonkey:(BOOL)aMonkey
{
    CCSprite *sprite = [CCSprite spriteWithFile:imageName];
    [self addChild:sprite z:1];
	
    b2BodyDef bodyDef;
    bodyDef.type = aStatic ? b2_staticBody : b2_dynamicBody;
    bodyDef.position.Set((position.x+sprite.contentSize.width/2.0f)/PTM_RATIO,
                         (position.y+sprite.contentSize.height/2.0f)/PTM_RATIO);
    bodyDef.angle = CC_DEGREES_TO_RADIANS(rotation);
    bodyDef.userData = sprite;
    b2Body *body = world->CreateBody(&bodyDef);
	
    b2FixtureDef boxDef;
    if (aMonkey)
    {
        boxDef.userData = (void*)1;
        [monkeys addObject:[NSValue valueWithPointer:body]];
    }
	
	if (aCircle)
    {
        b2CircleShape circle;
        circle.m_radius = sprite.contentSize.width/2.0f/PTM_RATIO;
        boxDef.shape = &circle;
    }
    else
    {
        b2PolygonShape box;
        box.SetAsBox(sprite.contentSize.width/2.0f/PTM_RATIO, sprite.contentSize.height/2.0f/PTM_RATIO);
        boxDef.shape = &box;
    }
	
    boxDef.density = 0.5f;
    body->CreateFixture(&boxDef);
	
    [trees addObject:[NSValue valueWithPointer:body]];
}

- (void)resetGame
{
	// Previous bullets cleanup
    if (bullets)
    {
        for (NSValue *bulletPointer in bullets)
        {
            b2Body *bullet = (b2Body*)[bulletPointer pointerValue];
            CCNode *node = (CCNode*)bullet->GetUserData();
            [self removeChild:node cleanup:YES];
            world->DestroyBody(bullet);
        }
        [bullets release];
        bullets = nil;
    }
	
    // Previous targets cleanup
    if (trees)
    {
        for (NSValue *bodyValue in trees)
        {
            b2Body *body = (b2Body*)[bodyValue pointerValue];
            CCNode *node = (CCNode*)body->GetUserData();
            [self removeChild:node cleanup:YES];
            world->DestroyBody(body);
        }
        [trees release];
        [monkeys release];
        trees = nil;
        monkeys = nil;
    }
	
    [self createBullets:5];
	[self createTargets];
	
    [self runAction:[CCSequence actions:
                     [CCMoveTo actionWithDuration:3.0f position:CGPointMake(-1400.0f, 0.0f)], 
                     [CCCallFuncN actionWithTarget:self selector:@selector(attachBullet)],
                     [CCDelayTime actionWithDuration:0.5f],
                     [CCMoveTo actionWithDuration:1.0f position:CGPointZero],
                     nil]];
}

- (void)createBullets:(int)count
{
    currentBullet = 0;
    CGFloat pos = 62.0f;
	
    if (count > 0)
    {
        // delta is the spacing between bananas
        // 62 is the position o the screen where we want the bananas to start appearing
        // 165 is the position on the screen where we want the bananas to stop appearing
        // 30 is the size of the banana
        CGFloat delta = (count > 1) ? ((165.0f - 62.0f - 30.0f) / (count - 1)) : 0.0f;
		
        bullets = [[NSMutableArray alloc] initWithCapacity:count];
        for (int i=0; i<count; i++, pos+=delta)
        {
            // Create the bullet
            //
            CCSprite *sprite = [CCSprite spriteWithFile:@"banana.png"];
            [self addChild:sprite z:2];
			
            b2BodyDef bulletBodyDef;
            bulletBodyDef.type = b2_dynamicBody;
            bulletBodyDef.bullet = true;
            bulletBodyDef.position.Set(pos/PTM_RATIO,(FLOOR_HEIGHT+15.0f)/PTM_RATIO);
            bulletBodyDef.userData = sprite;
            b2Body *bullet = world->CreateBody(&bulletBodyDef);
            bullet->SetActive(false);
			
			b2PolygonShape shape;
			shape.SetAsBox(sprite.contentSize.width/2.0f/PTM_RATIO, sprite.contentSize.height/2.0f/PTM_RATIO);
			
            /*b2CircleShape shape;
            shape.m_radius = 15.0/PTM_RATIO;
			*/
            b2FixtureDef ballShapeDef;
            ballShapeDef.shape = &shape;
            ballShapeDef.density = 0.8f;
            ballShapeDef.restitution = 0.2f;
            ballShapeDef.friction = 0.99f;
            bullet->CreateFixture(&ballShapeDef);
			
            [bullets addObject:[NSValue valueWithPointer:bullet]];
        }
    }
}

- (BOOL)attachBullet
{
    if (currentBullet < [bullets count])
    {
        bulletBody = (b2Body*)[[bullets objectAtIndex:currentBullet++] pointerValue];
        bulletBody->SetTransform(b2Vec2(160.0f/PTM_RATIO, (140.0f+FLOOR_HEIGHT)/PTM_RATIO), 0.0f);
        bulletBody->SetActive(true);
		
        b2WeldJointDef weldJointDef;
        weldJointDef.Initialize(bulletBody, armBody, b2Vec2(160.0f/PTM_RATIO, (140.0f+FLOOR_HEIGHT)/PTM_RATIO));
        weldJointDef.collideConnected = false;
		
        bulletJoint = (b2WeldJoint*)world->CreateJoint(&weldJointDef);
        return YES;
    }
	
    return NO;
}

- (void)resetBullet
{
    if ([monkeys count] == 0)
    {
        // game over
        [self performSelector:@selector(resetGame) withObject:nil afterDelay:2.0f];
    }
    else if ([self attachBullet])
    {
        [self runAction:[CCMoveTo actionWithDuration:2.0f position:CGPointZero]];
    }
    else
    {
        // We can reset the whole scene here
		[self performSelector:@selector(resetGame) withObject:nil afterDelay:2.0f];
    }
	
	checkCollision= NO;
}

- (void)ccTouchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
    if (mouseJoint != nil) return;

	NSSet *set= [event allTouches];
	
	if(set.count == 1) {
		UITouch *myTouch = [touches anyObject];
		CGPoint location = [myTouch locationInView:[myTouch view]];
		location = [[CCDirector sharedDirector] convertToGL:location];
		b2Vec2 locationWorld = b2Vec2(location.x/PTM_RATIO, location.y/PTM_RATIO);
	
		if (locationWorld.x < armBody->GetWorldCenter().x + 50.0/PTM_RATIO)
		{
			b2MouseJointDef md;
			md.bodyA = groundBody;
			md.bodyB = armBody;
			md.target = locationWorld;
			md.maxForce = 2000;
			
			mouseJoint = (b2MouseJoint *)world->CreateJoint(&md);
		}	
	}
}

- (void)ccTouchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
    if (mouseJoint == nil) return;
	
	NSSet *set= [event allTouches];
	
	if(set.count == 1) {
		UITouch *myTouch = [touches anyObject];
		CGPoint location = [myTouch locationInView:[myTouch view]];
		location = [[CCDirector sharedDirector] convertToGL:location];
		b2Vec2 locationWorld = b2Vec2(location.x/PTM_RATIO, location.y/PTM_RATIO);
	
		mouseJoint->SetTarget(locationWorld);
	}
}

- (void)handlePinchFrom:(UIPinchGestureRecognizer *)pinch {
	if(pinch.state == UIGestureRecognizerStateEnded)
    {	
        currentScale = pinch.scale;
		ZB_last_posn = self.position;
    }
    else if((pinch.state == UIGestureRecognizerStateBegan || (pinch.state == UIGestureRecognizerStateChanged)) && currentScale != 0.0f)
    {	
        pinch.scale = currentScale;
    }
	
    if(pinch.scale != NAN && pinch.scale != 0.0)
    {
		[self zoomLayer:pinch.scale];
        //pinch.view.transform = CGAffineTransformMakeScale(pinch.scale, pinch.scale);
    }
}

// Zoom board
- (void)zoomLayer:(float)zoomScale {
	if ((self.scale*zoomScale) <= MIN_SCALE) {
		//zoomScale = MIN_SCALE/self.scale;
		
		return;
	}
	else if ((self.scale*zoomScale) >= MAX_SCALE) {
		//zoomScale =	MAX_SCALE/self.scale;
		
		return;
	}
	
	self.scale = zoomScale;
	
	//Move self so it appears to zoom from the current center of the screen.
	self.position = ccp(ZB_last_posn.x * zoomScale, ZB_last_posn.y * zoomScale);
	ZB_last_posn = self.position;
	
}

- (void)handlePanFrom:(UIPanGestureRecognizer *)recognizer {
	if(recognizer.state == UIGestureRecognizerStateBegan) {
        ZB_last_posn = self.position;
    }
	if (recognizer.state == UIGestureRecognizerStateChanged) {
		CGPoint translation = [recognizer translationInView:recognizer.view];
		translation.y = -1 * translation.y;
		[self moveBoard:translation from:ZB_last_posn];
	}
}

// Pan board
- (void)moveBoard:(CGPoint)translation from:(CGPoint)lastLocation {
	CGPoint target_position = ccpAdd(translation, lastLocation);
	
	CGSize size = [[CCDirector sharedDirector] winSize];
	
	// Insert routine here to check that target position is not out of bounds for your background
	// Remember that ZB_last_posn is a variable that holds the current position of self
	if(target_position.x <= 0 && target_position.y <= 0)
		self.position = target_position;
	else
		ZB_last_posn= lastLocation;
}

- (void)ccTouchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{	
    if (mouseJoint != nil)
    {
		if (armJoint->GetJointAngle() >= CC_DEGREES_TO_RADIANS(10))
		{
			releasingArm = YES;
		}
		
        world->DestroyJoint(mouseJoint);
        mouseJoint = nil;
    }
}

-(void) draw
{
	// Default GL states: GL_TEXTURE_2D, GL_VERTEX_ARRAY, GL_COLOR_ARRAY, GL_TEXTURE_COORD_ARRAY
	// Needed states:  GL_VERTEX_ARRAY, 
	// Unneeded states: GL_TEXTURE_2D, GL_COLOR_ARRAY, GL_TEXTURE_COORD_ARRAY
	glDisable(GL_TEXTURE_2D);
	glDisableClientState(GL_COLOR_ARRAY);
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	
	world->DrawDebugData();
	
	// restore default GL states
	glEnable(GL_TEXTURE_2D);
	glEnableClientState(GL_COLOR_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

}

-(void) tick: (ccTime) dt
{
	//It is recommended that a fixed time step is used with Box2D for stability
	//of the simulation, however, we are using a variable time step here.
	//You need to make an informed choice, the following URL is useful
	//http://gafferongames.com/game-physics/fix-your-timestep/
	
	int32 velocityIterations = 8;
	int32 positionIterations = 1;
	
	// Instruct the world to perform a single step of simulation. It is
	// generally best to keep the time step and iterations fixed.
	world->Step(dt, velocityIterations, positionIterations);

	
	//Iterate over the bodies in the physics world
	for (b2Body* b = world->GetBodyList(); b; b = b->GetNext())
	{
		if (b->GetUserData() != NULL) {
			//Synchronize the AtlasSprites position and rotation with the corresponding body
			CCSprite *myActor = (CCSprite*)b->GetUserData();
			myActor.position = CGPointMake( b->GetPosition().x * PTM_RATIO, b->GetPosition().y * PTM_RATIO);
			myActor.rotation = -1 * CC_RADIANS_TO_DEGREES(b->GetAngle());
		}	
	}
	
	// Arm is being released.
	if (releasingArm && bulletJoint)
	{
		// Check if the arm reached the end so we can return the limits
		if (armJoint->GetJointAngle() <= CC_DEGREES_TO_RADIANS(10))
		{
			releasingArm = NO;
			
			// Destroy joint so the bullet will be free
			world->DestroyJoint(bulletJoint);
			bulletJoint = nil;
			
			[self performSelector:@selector(resetBullet) withObject:nil afterDelay:5.0f];
		}
	}
	
	// Bullet is moving.
    if (bulletBody && bulletJoint == nil)
    {
        b2Vec2 position = bulletBody->GetPosition();
        CGPoint myPosition = self.position;
        CGSize screenSize = [CCDirector sharedDirector].winSize;

        // Move the camera.
        if (position.x > screenSize.width / 4.0f / PTM_RATIO || position.y > screenSize.height / PTM_RATIO)
        {
            myPosition.x = -MIN(screenSize.width*self.scale * 4.0f - screenSize.width*self.scale, position.x *self.scale* PTM_RATIO - screenSize.width *self.scale/ 4.0f);
			myPosition.y = -MAX(position.y*self.scale * PTM_RATIO - screenSize.width *self.scale/ 4.0f, 0);
			
            self.position = myPosition;
        }
		
		checkCollision= YES;
    }
    
	if(checkCollision) {
		// Check for impacts
		std::set<b2Body*>::iterator pos;
		for(pos = contactListener->contacts.begin(); 
			pos != contactListener->contacts.end(); ++pos)
		{
			b2Body *body = *pos;
        
			CCNode *contactNode = (CCNode*)body->GetUserData();
			CGPoint position = contactNode.position;
			[self removeChild:contactNode cleanup:YES];
			world->DestroyBody(body);
        
			[trees removeObject:[NSValue valueWithPointer:body]];
			[monkeys removeObject:[NSValue valueWithPointer:body]];
		
			CCParticleSun* explosion = [[CCParticleSun alloc] initWithTotalParticles:200];
			explosion.autoRemoveOnFinish = YES;
			explosion.startSize = 10.0f;
			explosion.speed = 70.0f;
			explosion.anchorPoint = ccp(0.5f,0.5f);
			explosion.position = position;
			explosion.duration = 1.0f;
			[self addChild:explosion z:11];
			[explosion release];
		}
	}
    
    // remove everything from the set
    contactListener->contacts.clear();
}

// on "dealloc" you need to release all your retained objects
- (void) dealloc
{
	// in case you have something to dealloc, do it in this method
	delete world;
	world = NULL;
	
	[bullets release];
	[trees release];
	[monkeys release];
	
	delete contactListener;
    contactListener = NULL;
	
	delete m_debugDraw;

	// don't forget to call "super dealloc"
	[super dealloc];
}
@end
