//
//  SBAppDelegate.m
//  SimpleBrowser
//
//  Created by Árpád Tamási on 2012.06.04..
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "SBAppDelegate.h"

@implementation SBAppDelegate
@synthesize backButton;
@synthesize forwardButton;
@synthesize progress;

@synthesize window;
@synthesize navbar;
@synthesize webview;



- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
    [navbar setStringValue:@"http://nyelvek.inf.elte.hu/leirasok/Objective_C/"];
    [self go: navbar];
}

- (IBAction)go:(id)sender {
    WebFrame *mainFrame = [webview mainFrame];
    NSURLRequest *request = [NSURLRequest requestWithURL:[NSURL URLWithString:[navbar stringValue]]]; 
    [mainFrame loadRequest: request];
}

/**
 Frame Load Delegate
 **/
 
/*!
 Keretletöltés kezdete.
 Most a navbar-ba beírjuk a frissen töltött címet
 A bitkolbászt elindítjuk
*/
- (void)webView:(WebView *)sender didStartProvisionalLoadForFrame:(WebFrame *)frame {
    NSLog(@"Keret letöltés indul: %@", [frame name]);
    if(![frame parentFrame]) {
        NSString *url = [[[[frame provisionalDataSource] request] URL] absoluteString];
        [navbar setStringValue:url];
        [progress startAnimation:frame];
    }
}

/*!
  Keret címe megérkezett
*/
- (void)webView:(WebView *)sender didReceiveTitle:(NSString *)title forFrame:(WebFrame *)frame
{
    NSLog(@"Keret cím megérkezett: %@ cím: %@", [frame name], title);

    // Report feedback only for the main frame.
    if (frame == [sender mainFrame]){
        [[sender window] setTitle:title];
    }
}

/*!
 Keret letöltés vége 
 History előre-hátra engedélyezése és tiltása
 A bitkolbászt leállítjuk
*/
- (void)webView:(WebView *)sender didFinishLoadForFrame:(WebFrame *)frame
{
    NSLog(@"Keret letöltés kész: %@", [frame name]);

    if (frame == [sender mainFrame]){
        [backButton setEnabled:[sender canGoBack]];
        [forwardButton setEnabled:[sender canGoForward]];
        [progress stopAnimation:frame];
    }
}

/*!
 A szerver átirányított
*/ 
- (void)webView:(WebView *)sender didReceiveServerRedirectForProvisionalLoadForFrame:(WebFrame *)frame
{
    NSLog(@"Szerveroldali átirányítás: %@", [frame name]);
}

/*!
 Az ideiglenes letöltés sikertelem
*/
- (void)webView:(WebView *)sender didFailProvisionalLoadWithError:(NSError *)error forFrame:(WebFrame *)frame
{
    NSLog(@"Ideiglenes letöltés sikertelen: %@ hiba: %@", [frame name], [error localizedDescription]);
}

/*!
 A véglegesített letöltés sikertelem
 */
- (void)webView:(WebView *)sender didFailLoadWithError:(NSError *)error forFrame:(WebFrame *)frame
{
    NSLog(@"Végleges letöltés sikertelen: %@ hiba: %@", [frame name], [error localizedDescription]);
}

/*!
 Adat érkezett, letöltés véglegesíthető
*/
- (void)webView:(WebView *)sender didCommitLoadForFrame:(WebFrame *)frame
{
    NSLog(@"Keret letöltés véglegesítve: %@", [frame name]); 
}

/*!
 Keret scroll pozíciója megváltozott
*/ 
- (void)webView:(WebView *)sender didChangeLocationWithinPageForFrame:(WebFrame *)frame
{
    NSLog(@"Keret scroll pozíciója megváltozott: %@", [frame name]); 
}


/*!
 A keret záródik
*/
- (void)webView:(WebView *)sender willCloseFrame:(WebFrame *)frame
{
    NSLog(@"A keret záródik: %@", [frame name]); 
}



/**
 Resource Load Delegate
**/

/*!
 Új erőforrás-azonosító. Számozzuk, a következő üzenetekben a sorszámmal hivatkozunk
*/ 
 
- (id)webView:(WebView *)sender identifierForInitialRequest:(NSURLRequest *)request fromDataSource:(WebDataSource *)dataSource {
    id identifier = [NSNumber numberWithInt:resourceCount++];
    NSLog(@"Új erőforrás sorszám: %@, %@", [request URL], identifier);
    return identifier;
}

/*!
 Indul az erőforrás letöltése
*/
- (NSURLRequest *)webView:(WebView *)sender resource:(id)identifier willSendRequest:(NSURLRequest *)request redirectResponse:(NSURLResponse *)redirectResponse fromDataSource:(WebDataSource *)dataSource
{
    NSLog(@"Erőforrás letöltése indul: %@", identifier);
    return request;
}

/*!
 Indul az erőforrás letöltése
 */
- (void)webView:(WebView *)sender resource:(id)identifier didReceiveResponse:(NSURLResponse *)response fromDataSource:(WebDataSource *)dataSource
{
    NSLog(@"Adat jött erőforráshoz: %@", identifier);
}

/*!
 Megjött az erőforrás hossza
*/ 
- (void)webView:(WebView *)sender resource:(id)identifier didReceiveContentLength:(int)length fromDataSource:(WebDataSource *)dataSource
{
    NSLog(@"Megjött az erőforrás hossza: %@: %d", identifier, length);
}


/*!
 Erőforrás kész
*/ 
- (void)webView:(WebView *)sender resource:(id)identifier didFinishLoadingFromDataSource:(WebDataSource *)dataSource
{
    NSLog(@"Erőforrás kész: %@", identifier);   
}

/*!
 Erőforrás sikertelen
*/
- (void)webView:(WebView *)sender resource:(id)identifier didFailLoadingWithError:(NSError *)error fromDataSource:(WebDataSource *)dataSource
{
    NSLog(@"Erőforrás sikertelen: %@, hiba: %@", identifier, [error localizedDescription]);   
}
@end
