#import <Foundation/Foundation.h> 
#import <AppKit/AppKit.h> 
#import "Tilitoli.h"

@implementation Tilitoli 
 
 // inicializalo metodus
- initPont:(NSPoint)p andKep:(NSImage *)k andDimenzio:(int)d 
{ 
    NSRect keret; 
    int x, y; 
 
    keret.origin = p; 
    keret.size = [k size]; 
 
    keret.size.width -= ((int)keret.size.width)%d; 
    keret.size.height -= ((int)keret.size.height)%d; 
 
    [super initWithFrame:keret];

	kep = k; 
 
    darabMerete.width = keret.size.width/d; 
    darabMerete.height = keret.size.height/d; 
 
    if (darabMerete.width<20 || darabMerete.height<20){ 
        [NSException raise:NSInvalidArgumentException 
                      format:@"A valasztott kep tul kicsi egy %dx%d meretu tablahoz.", 
                      d, d]; 
    } 
 
    tdim = d;

	keverve = NO; 
    for(y=0; y<tdim; y++){ 
        for(x=0; x<tdim; x++){ 
            tabla[y][x] = y*tdim+x; 
        } 
    } 
    tabla[0][tdim-1] = -1; 
    uresX = tdim-1; uresY = 0; 
 
    return self; 
}


- (BOOL)acceptsFirstResponder 
{ 
    return YES; 
}

// visszaadja, hogy hany elem van a helyen
- (int)helyenVan 
{ 
    int x, y, osszesen = 0; 
 
    for(y=0; y<tdim; y++){ 
        for(x=0; x<tdim; x++){ 
            if(tabla[y][x] == y*tdim+x){ 
                osszesen++; 
            } 
        } 
    } 
 
    return osszesen; 
}

// megadott koordinataju elemet (es a kozte es az ures hely kozt levo elemeket)
// elmozgatja a negy lehetseges irany (fel, le, jobbra, balra) kozul a 
// megfelelobe
- mozgatX:(int)x Y:(int)y 
{ 
    int m; 
 
    if(x==uresX && y==uresY){ 
        NSBeep(); 
    } 
    else if(x==uresX){ 
        if(y>uresY){ // le
            for(m=uresY; m<y; m++){ 
                tabla[m][x] = tabla[m+1][x]; 
            } 
        } 
        else{ // fel
            for(m=uresY; m>y; m--){ 
                tabla[m][x] = tabla[m-1][x]; 
            } 
        } 
        tabla[y][x] = -1;
        uresY = y; 
    } 
    else if(y==uresY){ 
        if(x>uresX){ // left 
            for(m=uresX; m<x; m++){ 
                tabla[y][m] = tabla[y][m+1]; 
            } 
        } 
        else{ // right 
            for(m=uresX; m>x; m--){ 
                tabla[y][m] = tabla[y][m-1]; 
            } 
        } 
        tabla[y][x] = -1; uresX = x; 
    } 
    else{ 
        NSBeep(); 
    } 
 
    return self; 
}

// megkeveri a tablat a veletlenszeruen kivalasztott elemek es irany
// szerint. a keverest akkor tekintjuk helyesnek, ha a tablan levo elemek
// maximum negyede van jo helyen.
- kever:(id)sender 
{ 
    int maxJoHelyen = tdim*tdim/4, maxIteracio = tdim*tdim, i;

	do { 
        for(i=0; i<maxIteracio; i++){ 
            BOOL xTengelyMenten = lrand48()%2; 
            int m = lrand48()%(tdim-1); 
 
 
            if(xTengelyMenten==YES){ 
                if(m>=uresX){ 
                     m++; 
                } 
 
                [self mozgatX:m Y:uresY]; 
            } 
            else{ 
                if(m>=uresY){ 
                     m++; 
                } 
 
                [self mozgatX:uresX Y:m]; 
            } 
        } 
    } while([self helyenVan]>maxJoHelyen);

	keverve = YES; 
    [self setNeedsDisplay:YES]; 
 
    return self; 
}

// kirajzolja az egesz kivalasztott kepet, vagy a megfelelo darabokat,
// attol fuggoen, hogy a tabla kevert allapotban van-e.
- (void)drawRect:(NSRect)r 
{ 
    int x, y; 
 
    [[NSColor blackColor] set]; 
 
    if(keverve==NO){ 
        [kep compositeToPoint:r.origin 
               fromRect:r 
               operation:NSCompositeCopy]; 
        [NSBezierPath strokeRect:[self bounds]]; 
        return; 
    }

	for(y=0; y<tdim; y++){ 
        for(x=0; x<tdim; x++){ 
            int val = tabla[y][x], 
                px = val%tdim, py=val/tdim; 
            NSRect honnan, hova; 
 
            if(val==-1){ 
                continue; 
            } 
 
            honnan.origin.x = px*darabMerete.width; 
            honnan.origin.y = py*darabMerete.height; 
            honnan.size = darabMerete; 
 
            hova.origin.x = x*darabMerete.width; 
            hova.origin.y = y*darabMerete.height; 
            hova.size = darabMerete; 
 
 
            [kep compositeToPoint:hova.origin 
                    fromRect:honnan 
                    operation:NSCompositeCopy]; 
            [NSBezierPath strokeRect:hova]; 
        } 
    } 
}

// az eger lenyomasakor meghatarozza az eger poziciojanak megfelelo koordinatakat
// majd ez alapjan azt a darabot, amelyik felett az eger all, majd elmozgatja
// azt, ha lehet. ha az osszes darab a helyen van, akkor uzenetet ir ki
// a jatek vegerol.
- (void)mouseDown:(NSEvent *)e 
{ 
    NSPoint p; 
    int x, y; 
 
    p = [e locationInWindow]; 
    p = [self convertPoint:p fromView: nil]; 
 
    x = p.x/darabMerete.width; y = p.y/darabMerete.height; 
 
    if(keverve==NO || (x!=uresX && y!=uresY)){ 
        NSBeep(); 
        return; 
    } 
 
    [self mozgatX:x Y:y];

	if([self helyenVan]==tdim*tdim-1){ 
        keverve = NO; 
        [self display]; 
        NSRunAlertPanel(@"Gratulalok! Nyertel!", @"Sikerult kiraknod a kepet.", 
                         @"Ok", nil, nil); 
    } 
    else{ 
        [self setNeedsDisplay:YES]; 
    } 
} 
 
@end
