﻿#import <Foundation/Foundation.h>
#import <Foundation/NSObject.h>
#import <stdio.h>

#import "Complex.h"

//leszarmazott osztaly, ami megvaltoztatja a print muveletet
//es megvalositja az NSCopying protokollt
@interface ComplexB: Complex <NSCopying>
-(void) print;
@end

@implementation ComplexB
//print - mas formatumban
-(void) print {
    printf( "(%2.2f; %2.2f)", real, imaginary );
}

//NSCopying protokoll megvalostasahoz implementalnunk kell
//a copyWithZone muveletet
//zone definialja a felhasznalando memoriateruletet,
//ezt tovabb kell kuldenunk az allocWithZone uzenettel
-(Complex*) copyWithZone: (NSZone*) zone {
    Complex* r = [[Complex allocWithZone: zone] init];
    [r setReal: real];
    [r setImaginary: imaginary];
    return r;
}
@end

/*
Forditas:
gcc -o posing posing.m Complex.m -I /GNUstep/System/Library/Headers -L /GNUstep/System/Library/Libraries -lobjc -lgnustep-base -fconstant-string-class=NSConstantString 
*/
int main (int argc, const char * argv[])
{
    //lecseréljük a Complex osztályt, mostantól ComplexB fog létrejönni
    //poseAsClass -t csak akkor szabad meghívni, amikor még
    //nem küldtünk semmilyen üzenetet az osztálynak!
    [ComplexB poseAsClass: [Complex class]];
    
    Complex *c = [[Complex alloc] init];
    [c setReal: 1.2 andImaginary: 3.4];
    printf( "A komplex szam: " );
    [c print];
    printf( "\n" );
    [c release];
       
    Complex *c2 = [[Complex alloc] init];
    [c2 setReal: 1.2 andImaginary: 3.4];
    printf( "A komplex szam: " );
    [c2 print];
    printf( "\n" );
    
    //Protokoll ellenorzes
    if ( [c2 conformsToProtocol: @protocol( NSCopying )] == YES ) {
        printf( "Complex conforms to NSCopying\n" );
    
       //NSCopying hasznalat: masolhatoak az objektumok
       Complex *c3 = c2;
       printf( "Lemasolva: " );
       [c3 print];
       printf( "\n" );
    }

    [c2 release];
    return 0;
}

