﻿#import <Foundation/Foundation.h>
#import <unistd.h>

/*
gcc verzio: i686-apple-darwin10-gcc-4.2.1 (GCC) 4.2.1 (Apple Inc. build 5664)
 
build command: gcc -o runloop runloop.m -framework CoreFoundation -framework Foundation  -l objc
*/

/*
	A RunLoop obszerverenek a visszahivo metodusa
	Ha tortenik valami a RunLoopban, meghivodik a 
	fuggveny, es a parameterbol kideritheto, hogy mi tortent
*/
void MyRunLoopObserverCallback(
	CFRunLoopObserverRef observer,
	CFRunLoopActivity activity,
	void *info
   )
{
	NSString* message = nil;
	
	switch (activity)
	{
		case kCFRunLoopEntry:
			message = @"RunLoop entry";
			break;
			
		case kCFRunLoopBeforeTimers:
			message = @"RunLoop before timers";
			break;
			
		case kCFRunLoopBeforeSources:
			message = @"RunLoop before sources";
			break;
			
		case kCFRunLoopBeforeWaiting:
			message = @"RunLoop before waiting";
			break;
		
		case kCFRunLoopAfterWaiting:
			message = @"RunLoop after waiting";
			break;
			
		case kCFRunLoopExit:
			message = @"RunLoop exit";
			break;
			
		case kCFRunLoopAllActivities:
			message = @"RunLoop activities";
			break;
	}
	
	NSLog(@"RunLoop observer called: %@", message);
}

/*
	A szal osztaly, tartalmazza a szal main fuggvenyet, illetve
	a timer altal hivott fuggvenyt, illetve egy szamlalot,
	ami a timer alal hivott fuggveny lefutasait szamolja
*/
@interface MyThread : NSObject
{
	int counter;
}

-(void) threadMain;
-(void) doFireTimer;

@end

@implementation MyThread

/*
	A thread main fuggvenye, innen indul a masidik szal vezerlese
*/
- (void) threadMain
{
	/*Szamlalo nullazasa*/
	counter = 0;
	
	/*Lekerdezzuk a szalhoz tartozo RunLoop-ot*/
    NSRunLoop* myRunLoop = [NSRunLoop currentRunLoop];
 
	/*
		Letrehozunk egy RunLoopObserver objektumot (ez csak CF objektumkent
		hozhato letre), es beallitjuk neki visszahivo fuggvenynek a 
		MyRunLoopObserverCallback fuggvenyunket
	*/
    CFRunLoopObserverContext  context = {0, self, NULL, NULL, NULL};
    CFRunLoopObserverRef    observer = CFRunLoopObserverCreate(kCFAllocatorDefault,
            kCFRunLoopAllActivities, YES, 0, &MyRunLoopObserverCallback, &context);
 
	/*
		Hozzaadjuk az observert a RunLoophoz
	*/
    if (observer)
    {
        CFRunLoopRef    cfLoop = [myRunLoop getCFRunLoop];
        CFRunLoopAddObserver(cfLoop, observer, kCFRunLoopDefaultMode);
    }
 
    /*
		Keszitunk egy timert, ami a doFireTimer metodusunkat fogja hivogatni
	*/
    [NSTimer scheduledTimerWithTimeInterval:4.5 target:self
                selector:@selector(doFireTimer) userInfo:nil repeats:YES];
 
	/*
		10 iteracion keresztul fogjuk futtatni a RunLoop-unkat
	*/
    NSInteger loopCount = 10;
    do
    {
        /*
			A RunLoop 3 masodpercig var esemenyre, ha erkezik, futtatja
			majd megszakad a kovetkezo iteracioig
		*/
        [myRunLoop runUntilDate:[NSDate dateWithTimeIntervalSinceNow:3]];
        loopCount--;
    }
    while (loopCount);
}

- (void) doFireTimer
{
	NSLog(@"Timer fired %d times", ++counter);
}

@end

int main(int argc, char *argv[])
{
	/*
		Letrehozzuk a szal objektumot, majd elinditjuk azt
		A szemetgyujtes miatt nem kell felszabaditanunk az objektumokat
	*/
	MyThread* myThread = [[MyThread alloc] init];
    [NSThread detachNewThreadSelector:@selector(threadMain) toTarget:myThread withObject:nil];

    return 0;
}
